@@ -44,16 +44,15 @@ Config::initDefaultValues (void)
     m_display_style = DISPLAY_STYLE_TRADITIONAL;
     m_remember_every_input = FALSE;
     m_sort_option = SORT_BY_PHRASE_LENGTH_AND_PINYIN_LENGTH_AND_FREQUENCY;
-    m_show_suggestion = FALSE;
-    m_emoji_candidate = TRUE;
 
     m_shift_select_candidate = FALSE;
     m_minus_equal_page = TRUE;
-    m_comma_period_page = TRUE;
+    m_comma_period_page = FALSE;
     m_auto_commit = FALSE;
 
     m_double_pinyin = FALSE;
     m_double_pinyin_schema = DOUBLE_PINYIN_DEFAULT;
+    m_double_pinyin_show_raw = FALSE;
 
     m_init_chinese = TRUE;
     m_init_full = FALSE;
@@ -70,6 +69,16 @@ Config::initDefaultValues (void)
     m_both_switch = "";
     m_trad_switch = "<Control><Shift>f";
 
+    m_enter_key = TRUE;
+
+    m_lua_extension = TRUE;
+    m_english_input_mode = TRUE;
+    m_table_input_mode = TRUE;
+    m_use_custom_table = FALSE;
+    m_emoji_candidate = TRUE;
+    m_english_candidate = TRUE;
+    m_suggestion_candidate = FALSE;
+
     m_network_dictionary_start_timestamp = 0;
     m_network_dictionary_end_timestamp = 0;
 
