@@ -59,14 +59,13 @@ public:
     DisplayStyle displayStyle (void) const      { return m_display_style; }
     gboolean rememberEveryInput (void) const    { return m_remember_every_input; }
     sort_option_t sortOption (void) const       { return m_sort_option; }
-    gboolean showSuggestion (void) const        { return m_show_suggestion; }
-    gboolean emojiCandidate (void) const        { return m_emoji_candidate; }
     gboolean shiftSelectCandidate (void) const  { return m_shift_select_candidate; }
     gboolean minusEqualPage (void) const        { return m_minus_equal_page; }
     gboolean commaPeriodPage (void) const       { return m_comma_period_page; }
     gboolean autoCommit (void) const            { return m_auto_commit; }
     gboolean doublePinyin (void) const          { return m_double_pinyin; }
     DoublePinyinScheme doublePinyinSchema (void) const { return m_double_pinyin_schema; }
+    gboolean doublePinyinShowRaw (void) const   { return m_double_pinyin_show_raw; }
     gboolean initChinese (void) const           { return m_init_chinese; }
     gboolean initFull (void) const              { return m_init_full; }
     gboolean initFullPunct (void) const         { return m_init_full_punct; }
@@ -76,7 +75,14 @@ public:
     gboolean guideKey (void) const              { return m_guide_key; }
     gboolean auxiliarySelectKeyF (void) const   { return m_auxiliary_select_key_f; }
     gboolean auxiliarySelectKeyKP (void) const  { return m_auxiliary_select_key_kp; }
-    gboolean enterKey (void) const  { return m_enter_key; }
+    gboolean enterKey (void) const              { return m_enter_key; }
+    gboolean luaExtension (void) const          { return m_lua_extension; }
+    gboolean englishInputMode (void) const      { return m_english_input_mode; }
+    gboolean tableInputMode (void) const        { return m_table_input_mode; }
+    gboolean useCustomTable (void) const        { return m_use_custom_table; }
+    gboolean emojiCandidate (void) const        { return m_emoji_candidate; }
+    gboolean englishCandidate (void) const      { return m_english_candidate; }
+    gboolean suggestionCandidate (void) const   { return m_suggestion_candidate; }
 
     std::string mainSwitch (void) const         { return m_main_switch; }
     std::string letterSwitch (void) const       { return m_letter_switch; }
@@ -139,8 +145,6 @@ protected:
     DisplayStyle m_display_style;
     gboolean m_remember_every_input;
     sort_option_t m_sort_option;
-    gboolean m_show_suggestion;
-    gboolean m_emoji_candidate;
 
     gboolean m_shift_select_candidate;
     gboolean m_minus_equal_page;
@@ -149,6 +153,7 @@ protected:
 
     gboolean m_double_pinyin;
     DoublePinyinScheme m_double_pinyin_schema;
+    gboolean m_double_pinyin_show_raw;
 
     gboolean m_init_chinese;
     gboolean m_init_full;
@@ -163,6 +168,14 @@ protected:
 
     gboolean m_enter_key;
 
+    gboolean m_lua_extension;
+    gboolean m_english_input_mode;
+    gboolean m_table_input_mode;
+    gboolean m_use_custom_table;
+    gboolean m_emoji_candidate;
+    gboolean m_english_candidate;
+    gboolean m_suggestion_candidate;
+
     std::string m_main_switch;
     std::string m_letter_switch;
     std::string m_punct_switch;
