@@ -23,6 +23,7 @@
 
 #include "PYEditor.h"
 #include "PYLookupTable.h"
+#include "PYEnglishDatabase.h"
 
 namespace PY {
 
@@ -65,8 +66,6 @@ private:
     gboolean processEditKey(guint keyval);
     gboolean processPageKey(guint keyval);
 
-    gboolean train(const char *word, float delta);
-
 private:
     /* variables */
     LookupTable m_lookup_table;
