@@ -39,6 +39,9 @@ PhoneticEditor::PhoneticEditor (PinyinProperties &props,
     m_emoji_candidates (this),
 #ifdef ENABLE_CLOUD_INPUT_MODE
     m_cloud_candidates(this),
+#endif
+#ifdef IBUS_BUILD_ENGLISH_INPUT_MODE
+    m_english_candidates (this),
 #endif
     m_traditional_candidates (this, config)
 {
@@ -154,7 +157,7 @@ PhoneticEditor::processFunctionKey (guint keyval, guint keycode, guint modifiers
             reset ();
             return TRUE;
         default:
-            return TRUE;
+            return FALSE;
         }
     } else { /* ctrl key pressed. */
         switch (keyval) {
@@ -189,10 +192,10 @@ PhoneticEditor::processFunctionKey (guint keyval, guint keycode, guint modifiers
                 return TRUE;
             }
         default:
-            return TRUE;
+            return FALSE;
         }
     }
-    return TRUE;
+    return FALSE;
 }
 
 gboolean
@@ -251,6 +254,11 @@ PhoneticEditor::updateCandidates (void)
     if (!m_props.modeSimp ())
         m_traditional_candidates.processCandidates (m_candidates);
 
+#ifdef IBUS_BUILD_ENGLISH_INPUT_MODE
+    if (m_config.englishCandidate ())
+        m_english_candidates.processCandidates (m_candidates);
+#endif
+
     return TRUE;
 }
 
@@ -376,6 +384,11 @@ PhoneticEditor::selectCandidateInternal (EnhancedCandidate & candidate)
     case CANDIDATE_TRADITIONAL_CHINESE:
         return m_traditional_candidates.selectCandidate (candidate);
 
+#ifdef IBUS_BUILD_ENGLISH_INPUT_MODE
+    case CANDIDATE_ENGLISH:
+        return  m_english_candidates.selectCandidate (candidate);
+#endif
+
 #ifdef ENABLE_CLOUD_INPUT_MODE
     case CANDIDATE_CLOUD_INPUT:
         return m_cloud_candidates.selectCandidate (candidate);
@@ -409,6 +422,11 @@ PhoneticEditor::removeCandidateInternal (EnhancedCandidate & candidate)
     case CANDIDATE_TRADITIONAL_CHINESE:
         return m_traditional_candidates.removeCandidate (candidate);
 
+#ifdef IBUS_BUILD_ENGLISH_INPUT_MODE
+    case CANDIDATE_ENGLISH:
+        return m_english_candidates.removeCandidate (candidate);
+#endif
+
 #ifdef IBUS_BUILD_LUA_EXTENSION
     case CANDIDATE_LUA_TRIGGER:
         return m_lua_trigger_candidates.removeCandidate (candidate);
@@ -489,6 +507,9 @@ PhoneticEditor::selectCandidate (guint index)
     if (action & SELECT_CANDIDATE_UPDATE)
         update ();
 
+    if (action & SELECT_CANDIDATE_DIRECT_COMMIT)
+        directCommit (candidate.m_display_string.c_str ());
+
     return TRUE;
 }
 
@@ -505,6 +526,18 @@ PhoneticEditor::selectCandidateInPage (guint i)
     return selectCandidate (i);
 }
 
+void
+PhoneticEditor::directCommit (const gchar *str)
+{
+    if (G_UNLIKELY (m_text.empty ()))
+        return;
+
+    Text text (str);
+    commitText (text);
+
+    reset();
+}
+
 gboolean
 PhoneticEditor::removeCharBefore (void)
 {
